#ifndef QTELLASUB_H
#define QTELLASUB_H

#include <string>
#include <vector>
#include <list>
#include <set>

class QMenuBar;
class Connections;
class SharedFile;
class UploadManager;
class DownloadManager;
class IncomingConnect;
class QPopupMenu;
class HostCache;
class InterruptedDownloads;

class QTimer;
class ExecThread;

#include "qthread.h"
#include <qapplication.h>
#include <qstring.h>
#include <qfont.h>
#include <qpalette.h>

#include "Qtella.h"

#ifdef KDE
#include "kmenubar.h"
#include "kpopupmenu.h"
#endif

using namespace std;

class QtellaSub : public Qtella
{ 
    Q_OBJECT

 public:
    QtellaSub( QApplication* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~QtellaSub();

    #ifdef KDE
    KMenuBar*            menu;
    KPopupMenu*          _help;
    KPopupMenu*          _file;
    KPopupMenu*          _style;
    #endif

    #ifdef QT
    QMenuBar*            menu;
    QPopupMenu*          _help;
    QPopupMenu*          _file;
    QPopupMenu*          _style;
    #endif

    Connections*         connections;
    QTimer*              timer_connection;
    
    std::vector< pair<std::string, std::string> > _applications;
    std::vector<SharedFile*>  vSharedFiles;

    unsigned long             number_shared_files;
    double                    shared_size;

    static void               sighandler(int);
    void                      checkDirectories(std::string directory, std::vector<SharedFile*>* old);
    void                      removeFiles(const int size);
    void                      launchFile(std::string filename);
    void                      launchFileList(std::vector<std::string>& files);
    static QtellaSub*         getInstance();
    void                      search(std::string query);

    UploadManager*            upload_manager;
    DownloadManager*          download_manager;
    IncomingConnect*          _incoming_connect;
    HostCache*                _hostcache;
    InterruptedDownloads*     _interrupted_downloads;

    QPopupMenu*               _context_download;
    QPopupMenu*               _context_search;
    QPopupMenu*               _context_upload;
    QPopupMenu*               _context_shared;
    QPopupMenu*               _context_connect;

    QPalette                  _palette;
    QPalette                  _menu_palette;

 public slots:

    virtual void slotSharedFilesLaunch(QListViewItem* item);
    virtual void slotSharedFilesLaunch();
    virtual void slotAddApplication();
    virtual void slotRemoveApplication();
    virtual void slotEditApplication();
    virtual void slotChangeFinished();
    virtual void slotChangeInterrupted();
    virtual void slotChangeHostlist();
    virtual void slotChangeLogfile();
    virtual void slotSelectHostCacheLogFile();
    virtual void slotDeleteDownload();
    virtual void slotCloseSearch();
    virtual void slotLaunchInterruptedFile();
    virtual void slotDeleteInterruptedFile();
    virtual void slotSearchInterruptedFile();
    virtual void slotResumeInterruptedFile();
    virtual void slotAbortInterruptedFile();

    virtual void slotSearchStopDownload();
    virtual void slotSearchRemoveDownload();
    virtual void slotSearchDeleteDownload();

    void slotConnect();
    void slotDisconnect();
    void check_connections();
    void slotInfo();

    virtual void slotDownloadDoubleClicked(QListViewItem* item);
    virtual void slotDownloadShowContext(QListViewItem*, const QPoint&, int);
    virtual void slotLaunchSelectedDownloadItem();
    virtual void slotShowSearchContext(QListViewItem*, const QPoint&, int);
    virtual void slotShowUploadContext(QListViewItem*, const QPoint&, int);
    virtual void slotShowSharedContext(QListViewItem*, const QPoint&, int);
    virtual void slotSharedProperties();
    virtual void slotChangeInterface(const QString& text);
    virtual void slotClickedAutoConnect(int, QListBoxItem*, const QPoint&);
    virtual void slotConnectContext(QListViewItem*, const QPoint&, int);
    virtual void slotInterruptedFileContext(QListViewItem*, const QPoint&, int);
    virtual void slotUpdateAutoConnect();
    virtual void slotRefreshSharedList();
    virtual void slotSearch();
    virtual void slotAddDirectory();
    virtual void slotRemoveDirectory();
    virtual void slotApplyConfig();
    virtual void slotStopSearch();
    virtual void slotCurrentChanged(QWidget*);
    virtual void slotAddHostAutoConnect();

    virtual void slotDownloadAll();
    virtual void slotSearchDoubleClicked(QListViewItem*);

    virtual void slotAddAutoConnect();
    virtual void slotRemoveAutoConnect();
    virtual void slotAddHost();
    virtual void slotAbortUpload();
    virtual void slotRemoveUpload();
    virtual void slotDeleteSharedFile();
    virtual void slotClearInactiveUploads();
    virtual void slotReadFile();

    virtual void slotHostCacheChangePort(int port);

    void         slotStylePlatinum();
    void         slotStyleWindows();
    void         slotStyleMotif();
    void         slotStyleMotifPlus();
    void         slotStyleCDE();
    void         slotStyleSGI();
    void         slotStyleKDE();
    void         slotStyle(int style);
    void         slotChangeFont();

 private:
    friend class Download;
    friend class Connections;
    friend class QueryHitEntry;

    std::set<QListViewItem*>  _search_item_set;

    void                      downloadItem(QListViewItem *item);
    void                      readConfiguration();
    void                      readHostList();

    std::string               _pref_if;
    std::string               _last_addr;

    std::string               _binary;
    std::string               conffile;
    int                       search_tabs;

    QMutex                    mutex_shared_files;
    QFont                     _current_font;
    static QtellaSub*         _instance;
};

#endif // QTELLASUB_H
