#include "InterruptedFile.h"
#include "StringManipulation.h"
#include <qfile.h>
#include <strstream>
#include <iostream>
#include <iomanip>

InterruptedFile::InterruptedFile(std::string file) : _filename(file), _found(0), _colored(0)
{
}

const std::string InterruptedFile::filename()
{
  std::string::size_type idx = _filename.find_last_of("/");
  std::string file;

  if(idx != std::string::npos)
    file = _filename.substr(idx+1);
  else
    file = _filename;

  if( (idx = file.find(".part_")) == std::string::npos ) return file;

  std::string::size_type idx2 = file.find_last_of(".");
  file.erase(idx, idx2-idx);
  return file;
}

const std::string InterruptedFile::ext()
{
  std::string::size_type idx = _filename.find_last_of(".");

  if(idx != std::string::npos)
    return _filename.substr(idx+1);
  else
    return _filename;
}

const std::string InterruptedFile::strsize(bool comma)
{
  std::string::size_type idx = _filename.find(".part_");

  if(idx == std::string::npos) return "";

  idx = _filename.find("_", idx+6);
  std::string::size_type idx2 = _filename.find("_", idx+1);

  if(comma)
    return StringManipulation::insertDelimiter(_filename.substr(idx+1, idx2-idx-1), ',', 3);
  else
    return _filename.substr(idx+1, idx2-idx2-1);
}

const unsigned long InterruptedFile::size()
{
  std::string s = strsize(false);
  std::strstream str;
  str << s << std::ends;
  unsigned long size;
  str >> size;
  return size;
}

const unsigned long InterruptedFile::read()
{
  QFile f(_filename.c_str());
  return f.size();
}

const unsigned long InterruptedFile::toread()
{
  unsigned long size;
  std::strstream str;
  str << strsize() << std::ends;
  str >> size;
  return (size - read());
}

const double InterruptedFile::progress()
{
  if(size() > 0)
    return
      static_cast<double>(100) * static_cast<double>(read()) / static_cast<double>(size());
  else
    return 0;
}

const std::string InterruptedFile::strprogress(int precision)
{
  std::strstream str;
  str.setf(std::ios::fixed);
  str << std::setprecision(precision) << progress() << "%" << std::ends;
  std::string s(str.str());
  return s;
}

const::std::string InterruptedFile::ip()
{
  std::string::size_type idx = _filename.find(".part_");

  if(idx == std::string::npos) return "";

  idx = _filename.find("_", idx+6);
  idx = _filename.find("_", idx+1);
  std::string::size_type idx2 = _filename.find("_", idx+1);

  return _filename.substr(idx+1, idx2-idx-1);
}

const bool InterruptedFile::isValid()
{
  return (_filename.find(".part_") != std::string::npos);
}


const std::string InterruptedFile::completeName()
{
  return _filename;
}


const void InterruptedFile::incFound()
{
  ++_found;
}


const void InterruptedFile::decFound()
{
  --_found;
}

const void InterruptedFile::setColored()
{
  _colored = 1;
}

const void InterruptedFile::clearColored()
{
  _colored = 0;
}

const int InterruptedFile::getColored()
{
  return _colored;
}

const int InterruptedFile::getFound()
{
  return _found;
}
